include("../lmm.jl")
using JLD, RCall
# need R packages lme4
R"""
library(lme4)
"""

# a = no. levels factor 1, b = no. levels in factor 2
a = b = 5
# m = no. of variance components
m = 4
# c = different no. obs. per combination of levels
cs = [5 10 20 50]
nc = length(cs)
# different variance component ratio  σ2[1] / σ2[end]
vcratio = [0 0.05 0.1 1 10 20]
nratio = length(vcratio)
# no. methods to compare
nmethod = 4 # MM, EM, FS, lme4
# no. simulations for each scenario
nsim = 50

result  = zeros(8, nmethod, nc, nratio)
σ2      = ones(m)
obj     = zeros(nsim, nmethod)
t       = zeros(nsim, nmethod)
iter    = ones(Int, nsim, nmethod)
sqerror = zeros(nsim, nmethod)
for i in 1:nc
    for j in 1:nratio
        println("(c, vcratio) = ", (cs[i], vcratio[j]))
        for k in 1:nsim
            srand(13j  + 11i + 7k)
            c = cs[i]
            n = a * b * c
            σ2[1] = σ2[2] = σ2[3]= σ2[end] * vcratio[j]
            X = ones(n, 1)
            β = [1.0]
            Z = Array{Matrix{Float64}}(3)
            Z[1] = kron(eye(a), ones(b * c))
            Z[2] = kron(ones(a), kron(eye(b), ones(c)))
            Z[3] = kron(eye(a * b), ones(c))
            V = Array{Matrix{Float64}}(m)
            for l = 1:m-1
                V[l] = Z[l] * Z[l]'
            end
            V[end] = eye(n)
            Ω = zeros(n, n)
            for l = 1:m
                Ω += σ2[l] * V[l]
            end
            y = X * β + cholfact(Symmetric(Ω))[:L] * randn(n)
            
            lmm_woodbury(y, X, Z; algo = :MM) # trigger compilation
            t[k, 1] = @elapsed βhat, σ2hat, iter[k, 1], obj[k, 1] = lmm_woodbury(y, X, Z; algo = :MM)
            sqerror[k, 1] = sqrt(sum((σ2hat - σ2).^2) / m)
            
            t[k, 2] = @elapsed βhat, σ2hat, iter[k, 2], obj[k, 2] = lmm_woodbury(y, X, Z; algo = :EM)
            sqerror[k, 2] = sqrt(sum((σ2hat - σ2).^2) / m)
            
            t[k, 3] = @elapsed βhat, σ2hat, iter[k, 3], obj[k, 3] = lmm_woodbury(y, X, Z; algo = :FS)
            sqerror[k, 3] = sqrt(sum((σ2hat - σ2).^2) / m)

            # call lmer function in R
            @rput a
            @rput b
            @rput c
            R"""
            Z1 <- rep(1:a, each = b * c)
            Z2 <- rep(rep(1:b, each = c), a)
            Z3 <- rep(1:(a*b), each = c)
            df <- data.frame(y=$y, X = $X, Z1 = Z1, Z2 = Z2, Z3 = Z3)
            ptm <- proc.time()
            model <- lmer(y ~ 0 + X + (1|Z1) + (1|Z2)+ (1|Z3),
                data = df, REML = FALSE, start = rep(1.0, 3))
            time_lmer <- proc.time() - ptm
            elapsed_lmer <- time_lmer[3]
            logL_lmer <- as.numeric(logLik(model))
            vc_lmer <- VarCorr(model)
            vc_lmer_res <- attr(vc_lmer, "sc")
            """
            @rget logL_lmer
            @rget vc_lmer
            @rget vc_lmer_res
            @rget elapsed_lmer
            t[k, 4] = elapsed_lmer
            obj[k, 4] = logL_lmer
            σ2hat = [vc_lmer[:Z1], vc_lmer[:Z2], vc_lmer[:Z3], vc_lmer_res]
            sqerror[k, 4] = sqrt(sum((σ2hat - σ2).^2) / m)
        end
        
        result[1, :, i, j] = mean(iter, 1)
        result[2, :, i, j] = std(iter, 1)
        t = t .* 1e3 # milisecond
        result[3, :, i, j] = mean(t, 1)
        result[4, :, i, j] = std(t, 1)
        result[5, :, i, j] = mean(obj, 1)
        result[6, :, i, j] = std(obj, 1)
        result[7, :, i, j] = mean(sqerror, 1)
        result[8, :, i, j] = std(sqerror, 1)
    end
end

# machine information
juliainfo = IOBuffer()
versioninfo(juliainfo)
juliainfo = String(take!(juliainfo))
print(juliainfo)
R"""
rinfo = sessionInfo()
"""
@rget rinfo

# save workspace
@save "anova_sim_result.jld"
