include("../lmm.jl")
include("../vc.jl")

srand(123)

# a = no. levels factor 1, b = no. levels in factor 2
# c = no. obs. per combination of levels
a, b, c = 5, 5, 50
n = a * b * c
# set variance components
m = 4 
vcratio = 1
σ2 = ones(m)
σ2[1] = σ2[2] = σ2[3] = σ2[end] * vcratio
X = ones(n, 1)
β = [1.0]
Z = Array{Matrix{Float64}}(3)
Z[1] = kron(eye(a), ones(b * c))
Z[2] = kron(ones(a), kron(eye(b), ones(c)))
Z[3] = kron(eye(a * b), ones(c))
V = Array{Matrix{Float64}}(m)
for l in 1:m-1
    V[l] = Z[l] * Z[l]'
end
V[end] = eye(n)
Ω = zeros(n, n)
for l = 1:m
    Ω += σ2[l] * V[l]
end
y = vec(X * β) + cholfact(Symmetric(Ω))[:L] * randn(n)

# @code_warntype lmm_woodbury(y, X, Z, false, 1000, 1e-8, X \ y, ones(m), :MM)
# gc()
# Profile.clear()
# @profile lmm_woodbury(y, X, Z, false, 1000, 1e-8, X \ y, ones(m), :EM)
# Profile.print(format=:flat)

# @code_warntype vc(y, X, V, false, 1000, 1e-8, X \ y, ones(m), :FS)
# Profile.clear()
# @profile vc(y, X, V, false, 1000, 1e-8, X \ y, ones(m), :EM)
# Profile.print(format=:flat)

lmm_woodbury(y, X, Z; algo=:MM, maxiter=5) # trigger compilation
vc(y, X, V; algo=:MM, maxiter=5) # trigger compilation

println("MM using Woodbury:")
@time βhat, σ2hat, iter, obj = lmm_woodbury(y, X, Z; 
    verbose = false, maxiter = 1000, funtol = 1e-8, β = X \ y, σ2 = ones(m), algo = :MM)
@show βhat, σ2hat, iter, obj

println("MM without using Woodbury:")
@time βhat, σ2hat, iter, obj = vc(y, X, V;
    verbose = false, maxiter = 1000, funtol = 1e-8, β = X \ y, σ2 = ones(m), algo = :MM)
@show βhat, σ2hat, iter, obj

println("EM using Woodbury:")
@time βhat, σ2hat, iter, obj = lmm_woodbury(y, X, Z; 
    verbose = false, maxiter = 1000, funtol = 1e-8, β = X \ y, σ2 = ones(m), algo = :EM)
@show βhat, σ2hat, iter, obj

println("EM without using Woodbury:")
@time βhat, σ2hat, iter, obj = vc(y, X, V;
    verbose = false, maxiter = 1000, funtol = 1e-8, β = X \ y, σ2 = ones(m), algo = :EM)
@show βhat, σ2hat, iter, obj

println("FS using Woodbury:")
@time βhat, σ2hat, iter, obj = lmm_woodbury(y, X, Z; 
    verbose = false, maxiter = 1000, funtol = 1e-8, β = X \ y, σ2 = ones(m), algo = :FS)
@show βhat, σ2hat, iter, obj

println("FS without using Woodbury:")
@time βhat, σ2hat, iter, obj = vc(y, X, V;
    verbose = false, maxiter = 1000, funtol = 1e-8, β = X \ y, σ2 = ones(m), algo = :FS)
@show βhat, σ2hat, iter, obj
